/***************************************************************************
 * IoT Smart Street Light Control For Detecting Moving Object              *
 *                                                                         *
 * To control switching on and off light based on motion at surrounding    * 
 * and also the light intensity                                            *
 *                                                                         *
 * Created by: Johannes Marquart (2018) for ESP32 FischerTechnik Control   *
 *                                                                         *
 * Edited by: Nabilah Binti Salehuddin                                     *
 * Latest Update: 21 February 2019                                         *
 *                                                                         *
 ***************************************************************************/
//Wifi-Libs
#include <WiFi.h>
#include <WiFiClient.h>

//WebSocket-Libs
#include "WebSocketServer.h"

//Display-Libs
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
//#include <Wire.h>  //only for newer versions of Adafruit SSD1306 (1.2 or newer)


//display object with reset-pin
#define OLED_RESET 4  //reset-pin for Display
Adafruit_SSD1306 display(OLED_RESET); //creating Adafruit...-object display
//Adafruit_SSD1306 display(128, 64, &Wire, OLED_RESET); //only for newer versions of Adafruit SSD1306 (1.2 or newer)

//server objects
WiFiServer server(80);  //server-port
WebSocketServer webSocketServer;

unsigned int LDRSensor = 35; // select the input pin for the LDR
int LED1 = 4;    // select the pin for the LED
int PIRSensor = 13; //select input pin for PIR

bool connectedToNetwork = false;  //flag for checking if connection was lost

void setup()
{
  //setting up serial-connection
  Serial.begin(115200);

  //setting up display
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x32)
  display.display();  //showing the Adafruit logo (initialized graphic)
  display.clearDisplay();
  display.setTextColor(WHITE);
  delay(10);

  //declare input and output
  pinMode (LDRSensor, INPUT);
  pinMode (PIRSensor, INPUT);
  pinMode(LED1, OUTPUT);
  digitalWrite(LED1,LOW);

  //connecting to existing wifi
  char ssid[] = "HIT-FRITZBOX-7490";
  char password[] = "63601430989011937932";

  Serial.print("Connecting to ");
  Serial.println(ssid);

  display.print("Connecting to ");
  display.println(ssid);
  display.display();
  
  WiFi.mode(WIFI_STA);  //client mode (station mode (1)) -> connecting to an existing network
  WiFi.begin(ssid, password); //start wifi-module, according to mode: connect to existing network
  int i = 0;  //counts connection time
  while (WiFi.status() != WL_CONNECTED) //as long as wifi is not connected (WL_CONNECTED = 3, WL_DISCONNECTED = 6)
  {
    delay(500);
    //Serial.println(WiFi.status());
    Serial.print(".");
    if (10 == i)  //if no connection after some time (i*delay) restart esp or wlan (else it may take forever)
    {
      
      Serial.println("Connection Failed! Restart Wifi..."); //signal to reader via serial
      display.println("Connection Failed! Restart Wifi...");  //signal to reader via display
      display.display();
      delay(10);
      WiFi.reconnect(); //restart wifi
      delay(1000);  //wait for short time
      //ESP.restart();
      i = 0;  //reset time-counter
    }
    else  //if counter has not reached 10 increase counter and wait a little longer
    {
      i++;
    }
  }

  //after successful connection:
  connectedToNetwork = true;
  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  
  //Sending IP-Informations to display:
  display.clearDisplay();
  display.setCursor(0, 0);
  display.println("WiFi connected.");
  display.print("IP adr: ");
  display.println(WiFi.localIP());
  display.display();

  server.begin();
  delay(100);
}

void loop()
{

  WiFiClient client = server.available();
  unsigned long connectionTime = 0; //just out of interest: measures time untill network-connection is lost

  if (client.connected() && webSocketServer.handshake(client))
  {
    Serial.print("Client connected: ");
    Serial.println(client.remoteIP());

    String data;  //stores incoming data from websocket
    //int i = 0;  //unused
    bool serverIdle = false;  //stores if server is idle and awaits new data from websocket
    unsigned long timeOld = millis();
    connectionTime = millis();

    bool buttonPressed[2] = { false, false }; //stores current state of buttons, to detect button-state-change

    while (client.connected())
    {
      int PIRSensorValue = digitalRead(PIRSensor);
      int LDRSensorValue = analogRead(LDRSensor);
    
      /*
      //can be used for server to check if connection to client is still active
      if (millis() - timeOld > 4000)
      {
        webSocketServer.sendPing("--Heartbeat--");
      }
      */
      data = webSocketServer.getData(); //get webSocket-Data from Buffer (may be empty)
      delay(10); // Delay needed for receiving the data correctly
      
/*******************************************************AUTO SWICTHING LAMP BASED ON MOTION AND LIGHT INTENSITY*********************************************************************/
      //Condition for auto switching lamp
     
     if (LDRSensorValue < 2000)
     {
      if (PIRSensorValue == 1)
      {
      webSocketServer.sendData("A0");//sending message to client
      digitalWrite(LED1,HIGH);
      Serial.println ("LED ON");
      }
      else
      {
        webSocketServer.sendData("A1");//sending message to client
        digitalWrite(LED1, LOW);
        Serial.println ("LED OFF");
        }
     } 
/****************************************************************************************************************************************************************************/
      if (data.length() > 0)  //message is received
      {
        serverIdle = false; //server is busy, reset when data is empty
        
        Serial.println(data); //show recieved text on console
        
        Serial.println(millis() - timeOld); //time differences, for performance measurements
        timeOld = millis();

        while (data.length() > 0)
        {
          switch (data.charAt(0))
          {
          case 'D': //Handshake- and echo-message is printed on display and echoed back to client
            //show received text on display
            display.setCursor(0, 16);
            display.fillRect(0, 16, 128, 32, BLACK);  //clear required text-area
            display.println(data.substring(2)); //print timestamp (without identifier "D:")
            display.display();  //print everything on display
            webSocketServer.sendData(data); //send data-block back to client
            delay(10);
            break;

/************************************************************************************* MANUAL SWICTHING LAMP *****************************************************************/            
          case 'S': //Change lamp state ON/OFF manually
           {
            int LampState = data.substring(data.indexOf(':') + 1).toInt(); //getting message from client
            if (LampState == 1)
            {
             digitalWrite(LED1,HIGH);
             Serial.println ("LED ON");
            }
            if (LampState ==0)
            {
              digitalWrite(LED1, LOW);
              Serial.println ("LED OFF");
            }
            break;
           }

            default:  //in case of unknown data
            Serial.println("Wrong data-string");
            break;
          }

          //after handling data-block, remove it if possible
          if (0 <= data.indexOf(';')) 
          {
            data.remove(0, data.indexOf(';') + 1);  //remove handled data-block up to next ";"
          }
          else
          {
            data = "";
          }
        }
      }
      else //no data from websocket-buffer
      {
        if (!serverIdle)  //server-idle message not sent yet
        {
          serverIdle = true;
          webSocketServer.sendData("--serverIdle--");
        }
        delay(100); //wait for some time (to reduce cpu-usage)
      }

/*************************************************************************************LDR LIGHT INTENSITY READING********************************************************************/   
           //Display level of light intensity
            if (LDRSensorValue >= 4000)
            {
              webSocketServer.sendData("L0");//sending message to client
              Serial.println("High");
            }
            else if (LDRSensorValue >= 3000)
            {
              webSocketServer.sendData("L1");//sending message to client
              Serial.println("Normal Daylight");
            }
            else if (LDRSensorValue < 2000)
            {
              webSocketServer.sendData("L2");//sending message to client
              Serial.println("Low");
            }
            delay(500);
            
/*****************************************************************************************PIR SENSOR READING***********************************************************************/
      //check if PIR sensor detects motion and send change
      if (PIRSensorValue == 0)
      {
        webSocketServer.sendData("P0");//sending message to client
        Serial.println("No Motion");
      }
     else if (PIRSensorValue == 1)
      {
        webSocketServer.sendData("P1");//sending message to client
        Serial.println("Motion Detected");
      }
      delay(500);
    }
    
    Serial.println("Client disconnected");
    display.println("Client disconnected");
    display.display();
    //stop actors after client disconnected
    
    delay(1000);
  }

  //if connection to network is lost - print time since start of connection
  if ((true == connectedToNetwork) && (WL_CONNECTED != WiFi.status()))
  {
    Serial.println("Connection to Network lost after: ");
    Serial.println(millis() - connectionTime);
    display.println("Connection to Network lost after: ");
    display.println(millis() - connectionTime);
    display.display();
    connectedToNetwork = false;
  }

  delay(100);
}
